<?php

namespace Drupal\beehotel_pricealterator;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class PriceAlteratorBaseTable.
 *
 * @package Drupal\beehotel_pricealterator\Services
 *   this is part of the beehotel package
 */
class PreAlter {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * A construct method for the PreAlter class.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   Current user.
   */
  public function __construct(AccountInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * Price base table.
   *
   * @return array
   *   get the price base table.
   */
  public function baseTable($data) {

    $config = \Drupal::config('beehotel_pricealterator.settings');
    $util = \Drupal::service('beehotel_pricealterator.util');
    $days = $util->days();
    $basetable = [];
    foreach ($days as $d => $label) {
      $tmp = $config->get($data['nid'] . "_" . $d . "_high");
      if (isset($tmp)) {
        $basetable[$data['nid']]['high'][$d] = $tmp;
      }
      else {
        // Data is missing.  Redirect to base table.
        $tmp = "no value for:";
        $tmp .= "nid:" . $data['nid'] . ", ";
        $tmp .= "day:" . $d . ", ";
        $tmp .= "season: high.";
        $tmp = \Drupal::messenger()->addWarning($tmp);
        $url = Url::fromRoute('entity.node.canonical', ['node' => $data['nid']]);
        new RedirectResponse($url->toString());
      }

      $tmp = $config->get($data['nid'] . "_" . $d . "_low");
      if (isset($tmp)) {
        $basetable[$data['nid']]['low'][$d] = $tmp;
      }
    }
    return $basetable;
  }

  /**
   * Get the season of the requested night.
   *
   * @return array
   *   get the season of the requested night.
   */
  public function season() {
    $module_handler = \Drupal::moduleHandler();
    // Get season.
    $season = $module_handler->invokeAll('beehotel_pricealterator_season', [[]]);
    if (!isset($season)) {
      return "\$season is required. Is at least beehotel_pricealterators_beehotel_pricealterator_season running?";
    }
    return reset($season);
  }

}
