<?php

namespace Drupal\beehotel_vertical\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\Component\Utility\Unicode;

/**
 * Provides route responses for the Vertical module.
 */
class Vertical extends ControllerBase {

  /**
   * Produces result.
   */
  public function result($data) {

    $data['timeranges'] = beehotel_vertical_timeranges();
    $data['table_label'] = $data['timeranges'][$data['range']]['label'];
    $data['rows'] = $this->rows($data);
    $data['header'] = $this->header($data);

    $output['table'] = [
      '#caption' => $data['table_label'],
      '#type' => 'table',
      '#attributes' => [
        'class' => ['vertical-table'],
      ],
      '#header' => $data['header'],
      '#rows' => $data['rows'],
      '#empty' => t('Your table is empty'),
    ];
    return \Drupal::service('renderer')->render($output);
  }

  /**
   * Produces header.
   */
  private function header($data) {

    $data['units'] = bee_hotel_get_units($options = ['status' => 1]);

    $header = [];
    $header[] = Markup::create('<strong>' . $this->t('Day') . '</strong>');

    if (!empty($data['units'])) {
      foreach ($data['units'] as $unit) {
        $header[] = Markup::create("<h3 class='unit'>" . $unit->GetTitle() . "</h3>");
      }
    }
    $data['header'] = $header;
    return $data['header'];

  }

  /**
   * Produces rows.
   */
  private function rows($data) {

    $data['timeranges'] = beehotel_vertical_timeranges();
    $data['rowsnumber'] = $this->rowsNumber();
    $data['units'] = bee_hotel_get_units($options = ['status' => 1]);

    // Produce a row for every requested day.
    for ($i = 0; $i <= $data['rowsnumber']; $i++) {
      $data['day']['timestamp'] = time() + ($i * (60 * 60 * 24));
      // @todo move this into a twig template.
      $data['day']['formatted'] =
        "<span class='day-of-the-week'>" .
          \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'D') . ",
        </span>" .

        "<span class='day-month'>" .
         \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'd M') . ",
        </span>" .

        "<span class='year'>" .
         \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'Y') . "
        </span>";

      $data['day']['d'] = \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'd');
      $data['day']['month'] = \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'n');
      $data['day']['m'] = \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'm');
      $data['day']['year'] = \Drupal::service('date.formatter')->format($data['day']['timestamp'], 'custom', 'Y');

      // ISO 8601.
      $data['day']['today']['ISO8601'] = $data['day']['year'] . "-" . $data['day']['m'] . "-" . $data['day']['d'];
      $data['day']['daybefore']['ISO8601'] = date('Y-m-d', strtotime("-1 day", strtotime($data['day']['today']['ISO8601'])));

      // Produce columns.
      $columns = [];

      // Column 1 = day.
      $columns[] = Markup::create('<strong>' . $data['day']['formatted'] . '</strong>');

      // Produce a column per unit.
      if (!empty($data['units'])) {
        foreach ($data['units'] as $unit_id => $unit) {
          $data['unit']['node'] = $unit;
          $data['unit']['id'] = $unit_id;
          $data = $this->cellContent($data);
          $columns[] = \Drupal::service('renderer')->render($data['cellcontent']);
        }
        $data['rows'][] = $columns;

      }
    }
    return $data['rows'];
  }

  /**
   * Get time range for current report.
   */
  public function requestedRange() {
    $request = \Drupal::request();
    $session = $request->getSession();
    $range = $session->get('beehotel_requested_range');
    return $range;
  }

  /**
   * Get rows number for current report.
   */
  private function rowsNumber() {
    $data['range'] = $this->requestedRange();
    $data['ranges'] = beehotel_vertical_timeranges();
    $data['rows'] = $data['ranges'][$data['range']]['rows'];
    return $data['rows'];
  }

  /**
   * Produce cell content (html + data).
   */
  private function cellContent($data) {
    $data = $this->typeofOccupacy($data);
    $data['cellcontent'] = $this->verticalOrderItem($data, $options = []);
    return $data;
  }

  /**
   * Check for event id existance in the montly BAT table.
   *
   *   Default template: bat-type-group-add-list.html.twig.
   */
  private function typeofOccupacy($data) {

    $database = \Drupal::database();
    $query = $database->select('bat_event_availability_daily_day_state', 'montlytable');

    // Fields.
    $query->addField('montlytable', 'd' . ($data['day']['d'] * 1), 'd' . ($data['day']['d'] * 1));

    // Conditions.
    $query->condition('montlytable.unit_id', $data['unit']['id'], '=');
    $query->condition('montlytable.year', $data['day']['year'], '=');
    $query->condition('montlytable.month', $data['day']['month'], '=');
    $query->condition('montlytable.d' . ($data['day']['d'] * 1), 0, '<>');
    $query->range(0, 1);

    $state_id = $query->execute()->fetchField(0);
    $data['tmp']['event_id'] = bee_hotel_get_bat_event_id($data);

    // Current.
    $data['occupancy']['current']['event']['id'] = $data['tmp']['event_id'];
    $data['occupancy']['current']['event']['state'] = $this->getEventState($state_id);
    $data['occupancy']['current']['order'] = bee_hotel_from_event_to_order($data['occupancy']['current']['event']['id']);

    // Given  ISO 8601/unit.
    $data['occupancy'][$data['day']['today']['ISO8601']][$data['unit']['id']]['event']['id'] = $data['tmp']['event_id'];
    $data['occupancy'][$data['day']['today']['ISO8601']][$data['unit']['id']]['event']['state'] = $this->getEventState($state_id);

    $data['tmp']['order'] = bee_hotel_from_event_to_order($data['occupancy']['current']['event']['id']);
    $data['occupancy'][$data['day']['today']['ISO8601']][$data['unit']['id']]['order'] = $data['tmp']['order'];
    return $data;
  }

  /**
   * Get state for a event?
   *
   * @todo move this into the bee hotel global Event class.
   */
  private function getEventState($state_id) {

    $database = \Drupal::database();
    $query = $database->select('states', 's');

    // Fields.
    $query->addField('s', 'blocking');
    $query->addField('s', 'name');
    $query->addField('s', 'color');
    $query->addField('s', 'calendar_label');
    $query->addField('s', 'id');

    // Conditions.
    $query->condition('s.id', $state_id, '=');

    $result = $query->execute()->fetch();
    return $result;
  }

  /**
   * Is this event blocking?
   *
   * @todo move this into the bee hotel global Event class.
   */
  private function eventIsBlocking($state_id) {
    $is_blocking = FALSE;
    $database = \Drupal::database();
    $query = $database->select('states', 's');

    // Fields.
    $query->addField('s', 'blocking');

    // Conditions.
    $query->condition('s.id', $state_id, '=');
    $query->range(0, 1);

    $result = $query->execute()->fetchField(0);

    return $is_blocking;
  }

  /**
   * Get info for a given order.
   */
  private function getOrderData($data, $options) {
    $tmp = [];
    $tmp['day'] = $options['day'];
    $tmp['unit'] = $data['unit']['id'];
    $tmp['currentday_order_id'] = $this->getCurrentDayOrderId($data, $tmp['unit']);
    $tmp['daybefore_order_id'] = $this->getDayBeforeOrderId($data, $tmp['unit']);

    $order = [];
    $order['total_price__number'] = 0;

    if (!empty($data['occupancy'][$tmp['day']][$tmp['unit']]) && !empty($data['occupancy'][$tmp['day']][$tmp['unit']]['order'])) {
      $order['order_id'] = $data['occupancy'][$tmp['day']][$tmp['unit']]['order']->order_id;
      $order['mail'] = $data['occupancy'][$tmp['day']][$tmp['unit']]['order']->mail;
      $order['total'] = $data['occupancy'][$tmp['day']][$tmp['unit']]['order']->total_price__number;
      $order['order_number'] = $data['occupancy'][$tmp['day']][$tmp['unit']]['order']->order_number;
      return $order;
    }
  }

  /**
   * Theme a td.
   */
  private function verticalTableTrTd($data, $options) {

    $a_attributes = $this->verticalTableTrTdAttributes($data, $options = ['div' => "a"]);
    $a_class = implode($a_attributes['class'], " ");
    $a_content = $this->verticalOrderItem($data, $options = ['div' => "a"]);

    $spacer_attributes = $this->verticalTableTrTdAttributes($data, $options = ['div' => "spacer"]);
    $spacer_class = implode($spacer_attributes['class'], " ");
    $spacer_content = "";

    $b_attributes = $this->verticalTableTrTdAttributes($data, $options = ['div' => "b"]);
    $b_class = implode($b_attributes['class'], " ");
    $b_content = $this->verticalOrderItem($data, $options = ['div' => "b"]);

    return [
      '#theme' => 'vertical_table_tr_td',
      '#a_class' => $a_class,
      '#a_content' => $a_content,
      '#spacer_class' => $spacer_class,
      '#spacer_content' => $spacer_content,
      '#b_class' => $b_class,
      '#b_content' => $b_content,
    ];

  }

  /**
   * Build up order info.
   */
  private function verticalOrderItem($data, $opt) {

    $tmp = [];
    $tmp['unit'] = $data['unit']['id'];
    $tmp['currentday_order_id'] = $this->getCurrentDayOrderId($data, $tmp['unit']);
    $tmp['daybefore_order_id'] = $this->getDayBeforeOrderId($data, $tmp['unit']);

    $order = [];

    if ($opt['div'] == "a") {
      $options = [
        'day' => $data['day']['daybefore']['ISO8601'],
        'div' => $opt['div'],
      ];
      $order = $this->getOrderData($data, $options);
    }

    elseif ($opt['div'] == "b") {
      if (($tmp['daybefore_order_id'] != $tmp['currentday_order_id'])) {
        $options = [
          'day' => $data['day']['today']['ISO8601'],
          'div' => $opt['div'],
        ];
        $order = $this->getOrderData($data, $options);
      }
    }

    $mail = $order['mail'] ?? '';
    $total = $order['total_price__number'] ?? 0;

    return [
      '#theme' => 'vertical_order_item',
      '#order_id' => $order['order_id'] ?? '',
      '#mail' => Unicode::truncate($mail, 8, FALSE, TRUE) ,
      '#total' => number_format($total),
      '#order_number' => $order['order_number'] ?? '',
    ];
  }

  /**
   * Produces html attributes for a td.
   */
  private function verticalTableTrTdAttributes($data, $options) {

    $attributes = ['class' => []];
    $tmp['unit'] = $data['unit']['id'];
    $tmp['currentday_order_id'] = $this->getCurrentDayOrderId($data, $tmp['unit']);
    $tmp['daybefore_order_id'] = $this->getDayBeforeOrderId($data, $tmp['unit']);

    if (empty($data['occupancy'][$data['day']['daybefore']['ISO8601']])) {
      $data['first_row_of_the_table'] = TRUE;
    }

    // A.
    if ($options['div'] == "a") {

      // A1. Same reservation today and yestarday.
      // covered by A2 and A3.
      // if ( $tmp['currentday_order_id'] > 0 &&.
      // ($tmp['currentday_order_id'] == $tmp['daybefore_order_id'])) {.
      // $attributes['class'][] = 'reservation';
      // }.
      // A2. Guest is checkin out today.
      if (!empty($tmp['daybefore_order_id']) && $tmp['currentday_order_id'] != $tmp['daybefore_order_id']) {
        $attributes['class'][] = 'reservation';
        $attributes['class'][] = 'checkout';
      }

      // A3. Today's reservation is the same as yestarday.
      if (!empty($tmp['daybefore_order_id']) && $tmp['currentday_order_id'] == $tmp['daybefore_order_id']) {
        $attributes['class'][] = 'reservation';
        $attributes['class'][] = 'same-as-daybefore';
      }

      // A4. Show Checkout on the first day in table.
    }

    // Spacer.
    if ($options['div'] == "spacer") {

      // SPACER1. We have some reservation today and today reservation
      // is not checkin in.
      if (!empty($data['occupancy']['current']) &&
        !empty($data['occupancy']['current']['order']) &&
        $data['occupancy']['current']['order']->checkin != $data['day']['today']['ISO8601']
      ) {
        $attributes['class'][] = 'occupied';
      }
    }

    // B.
    if ($options['div'] == "b") {

      // B1. We have a reservation today.
      if (!empty($data['occupancy']['current']) && !empty($data['occupancy']['current']['order'])) {

        // 0
        // dump ($data);exit;
        // .
        $attributes['class'][] = 'reservation';

        // B1.1. Today reservation is checkin today.
        if ($data['occupancy']['current']['order']->checkin == $data['day']['today']['ISO8601']) {
          $attributes['class'][] = 'checkin';
        }
      }
    }
    return $attributes;
  }

  /**
   * Get the current day for the current order.
   */
  private function getCurrentDayOrderId($data, $unit) {
    if (!empty($data)) {
      if (!empty($data['occupancy'])) {
        if (!empty($data['day']['today']['ISO8601'])) {
          if (!empty($data['occupancy'][$data['day']['today']['ISO8601']])) {
            if (!empty($unit)) {
              if (!empty($data['occupancy'][$data['day']['today']['ISO8601']][$unit]['order'])) {
                if (!empty($data['occupancy'][$data['day']['today']['ISO8601']][$unit]['order'])) {
                  return $data['occupancy'][$data['day']['today']['ISO8601']][$unit]['order']->order_id;
                }
              }
            }
          }
        }
      }
    }
  }

  /**
   * Get the day before of the current order.
   */
  private function getDayBeforeOrderId($data, $unit) {
    if (!empty($data)) {
      if (!empty($data['occupancy'])) {
        if (!empty($data['day']['daybefore']['ISO8601'])) {
          if (!empty($data['occupancy'][$data['day']['daybefore']['ISO8601']])) {
            if (!empty($unit)) {
              if (!empty($data['occupancy'][$data['day']['daybefore']['ISO8601']][$unit]['order'])) {
                return $data['occupancy'][$data['day']['daybefore']['ISO8601']][$unit]['order']->order_id;
              }
            }
          }
        }
      }
    }
  }

}
