<?php

namespace Drupal\beehotel_pricealterator;

use Drupal\Core\Session\AccountInterface;

/**
 * Class Alter in charge of the price alteration.
 *
 * @package Drupal\beehotel_pricealterator\Services
 */
class Alter {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Alter constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(AccountInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * Contains the alteration flow.
   */
  public function Alter($data) {

    $tempstore = \Drupal::service('tempstore.private')->get('beehotel_pricealterator');

    // 1. Get content.
    // 1a. Base Table with base price
    $basetable = \Drupal::service('beehotel_pricealterator.prealter')->BaseTable();

    // 1b. Normalise data
    $data = $this->Normalise($data);

    // 1c. Validate data
    $data = $this->Validate($data);

    // 1d. Available Alterators see......
    $pricealterator = \Drupal::service('plugin.manager.beehotel.pricealterator');
    $alterators = $pricealterator->SortedAlterators();

    // 1e. Enabled Alterators
    $alterators = $this->CheckStatus($alterators);
    $tempstore->set('beehotel_pricealterator_log', $alterators);

    /* 2. Loop per night
    Drupal Commerce calls Resolver once per order item (not once per night)
    We loop nights here, providing an average price
    to be multiplied by items (nights)
    @todo keep track of evety night price into \$data
     */
    for ($n = 0; $n < $data['nights_as_integer']; $n++) {
      $data['tmp'] = NULL;
      $data['tmp']['night_timestamp'] = $data['checkin_timestamp'] + (24 * 3600 * $n);
      $data['tmp']['night_date'] = date('Y-m-d', $data['tmp']['night_timestamp']);

      foreach ($alterators as $a) {
        $alterator = new $a['class']([], [], [], []);

        // 3. get price for every night
        $data = $alterator->alter($data, $basetable);

        if (isset($data['tmp']['price'])) {
          $price[] = $data['tmp']['price'];
        }
      }
    }

    // 4. calculate average price.
    $price = array_filter($price);
    if (count($price)) {
      $average = array_sum($price) / count($price);
    }

    // Store the average as price per item (night).
    $data['amount'] = $average;

    // 5. return average price.
    return $data;

  }

  /**
   * Validate data.
   *
   * Check name change to lowerCamel format.
   */
  private function checkStatus($alterators) {

    $a = [];

    // @todo move this into yml config.
    $legenda = [
      0 => "disabled",
      1 => "enabled",
      2 => "work in progress",
    ];

    foreach ($alterators as $item) {
      if ($item['status'] == 1) {
        $a[] = $item;
      }
    }

    return $a;

  }

  /**
   * Validate data.
   *
   * Check for duplication.
   */
  private function validate($data) {

    if ($data['adults'] < 1 && $data['adults'] > 3) {
      // @todo improve this check
      die("ERR 34wfr: check occupancy");
    }
    return $data;
  }

  /**
   * Normalize data.
   *
   * Check for duplication.
   */
  private function normalise($data) {

    $data['checkin_timestamp'] = strtotime($data['checkin']);
    $data['checkout_timestamp'] = strtotime($data['checkout']);
    // 1 day = 86400.
    $data['nights'] = ($data['checkout_timestamp'] - $data['checkin_timestamp']) / 86400;
    $data['nights_as_integer'] = number_format($data['nights']);
    $data['checkin_timestamp'] = strtotime($data['checkin']);
    // 0 = Monday.
    $data['day_of_the_week'] = date("N", $data['checkin_timestamp']);
    $tmp = $data['entity']->get('sku')->value;
    $data['id_room'] = strtoupper($tmp[0] . $tmp[1]);

    return $data;

  }

  /**
   * How to send this output to the debug block?
   *
   * @todo implement the feature.
   */
  public function beehotelLog($data, $context) {}

}
